﻿#region Using ステートメント

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

#endregion

namespace ShaderInstancingTest02.GameDebug
{
    /// <summary>
    /// デバッグ用のグラフィクスコンテントを格納する為のデバッグ用マネージャー
    /// </summary>
    public class DebugManager : DrawableGameComponent
    {
        #region プロパティ

        /// <summary>
        /// デバッグ用コンテントマネージャーの取得
        /// </summary>
        public ContentManager Content { get; private set; }

        /// <summary>
        /// デバッグ用SpriteBatchの取得
        /// </summary>
        public SpriteBatch SpriteBatch { get; private set; }

        /// <summary>
        /// 白テクスチャの取得
        /// </summary>
        public Texture2D WhiteTexture { get; private set; }

        /// <summary>
        /// デバッグ用フォント
        /// </summary>
        public SpriteFont DebugFont { get; private set; }

        /// <summary>
        /// プレゼン用フォントの取得
        /// </summary>
        public SpriteFont PresentFont { get; private set; }

        #endregion

        #region 初期化

        public DebugManager( Game game )
            : base( game )
        {
            // サービスとして登録する
            Game.Services.AddService( typeof( DebugManager ), this );

            Content = new ContentManager( game.Services );
            Content.RootDirectory = "Content/Debug";

            // このコンポーネント自体はUpdate、Drawが呼ばれる必要はない
            this.Enabled = false;
            this.Visible = false;
        }

        protected override void LoadContent()
        {
            // デバッグ用コンテントの読み込み
            SpriteBatch = new SpriteBatch( GraphicsDevice );

            WhiteTexture = Content.Load<Texture2D>( "WhiteTexture" );
            DebugFont = Content.Load<SpriteFont>( "DebugFont" );
            PresentFont = Content.Load<SpriteFont>( "PresentFont" );

            base.LoadContent();
        }

        #endregion
    }
}