﻿#region Using ステートメント

using System;
using Microsoft.Xna.Framework;

#endregion

namespace ShaderInstancingTest02
{
    public static class FastMath
    {
        /// <summary>
        /// ラジアン角度を0～TwoPiの範囲に正規化する
        /// </summary>
        /// <param name="value"></param>
        /// <remarks>
        /// x86系のCPUの場合、SinやCosに0～TwoPi以外の極端に大きいまたは小さい値を
        /// 使って計算すると時間が掛かるのと、テイラー展開する場合にも
        /// 角度の値は0～TwoPiの間に限定されていると計算しやすいので正規化する。
        /// </remarks>
        public static float NormalizeRotation( float value )
        {
            // 剰余算(%)を使った方が簡潔に書けるが、元々の角度が0～TwoPiに近い値の場合、
            // 以下の方法の方が速い
            while ( value < 0.0f )
                value += MathHelper.TwoPi;

            while ( value > MathHelper.TwoPi )
                value -= MathHelper.TwoPi;

            return value;
        }
    }
}
